@@ -7,10 +7,11 @@ from typing import Optional
 
 import async_timeout
 
+from . import DeviceData
 from .api import COMMANDS_SECURED, QUEUE_TIMEOUT, RoborockClient
-from .containers import RoborockLocalDeviceInfo
 from .exceptions import CommandVacuumError, RoborockConnectionException, RoborockException
-from .roborock_message import AP_CONFIG, RoborockMessage, RoborockParser
+from .protocol import AP_CONFIG, MessageParser
+from .roborock_message import RoborockMessage
 from .roborock_typing import CommandInfoMap, RoborockCommand
 from .util import get_running_loop_or_create_one
 
@@ -18,10 +19,12 @@ _LOGGER = logging.getLogger(__name__)
 
 
 class RoborockLocalClient(RoborockClient, asyncio.Protocol):
-    def __init__(self, device_info: RoborockLocalDeviceInfo):
-        super().__init__("abc", device_info)
+    def __init__(self, device_data: DeviceData):
+        if device_data.host is None:
+            raise RoborockException("Host is required")
+        super().__init__("abc", device_data)
         self.loop = get_running_loop_or_create_one()
-        self.ip = device_info.network_info.ip
+        self.host = device_data.host
         self._batch_structs: list[RoborockMessage] = []
         self._executing = False
         self.remaining = b""
@@ -32,8 +35,7 @@ class RoborockLocalClient(RoborockClient, asyncio.Protocol):
         if self.remaining:
             message = self.remaining + message
             self.remaining = b""
-        (parser_msg, remaining) = RoborockParser.decode(message, self.device_info.device.local_key)
-        self.remaining = remaining
+        parser_msg, self.remaining = MessageParser.parse(message, local_key=self.device_info.device.local_key)
         self.on_message_received(parser_msg)
 
     def connection_lost(self, exc: Optional[Exception]):
@@ -46,17 +48,19 @@ class RoborockLocalClient(RoborockClient, asyncio.Protocol):
         async with self._mutex:
             try:
                 if not self.is_connected():
+                    self.sync_disconnect()
                     async with async_timeout.timeout(QUEUE_TIMEOUT):
-                        _LOGGER.info(f"Connecting to {self.ip}")
+                        _LOGGER.info(f"Connecting to {self.host}")
                         self.transport, _ = await self.loop.create_connection(  # type: ignore
-                            lambda: self, self.ip, 58867
+                            lambda: self, self.host, 58867
                         )
-                        _LOGGER.info(f"Connected to {self.ip}")
+                        _LOGGER.info(f"Connected to {self.host}")
             except Exception as e:
-                raise RoborockConnectionException(f"Failed connecting to {self.ip}") from e
+                _LOGGER.warning(f"Failed connecting to {self.host}: {e}")
+                raise RoborockConnectionException(f"Failed connecting to {self.host}") from e
 
     def sync_disconnect(self) -> None:
-        if self.transport and not self.loop.is_closed():
+        if self.transport and self.loop.is_running():
             self.transport.close()
 
     async def async_disconnect(self) -> None:
@@ -73,18 +77,15 @@ class RoborockLocalClient(RoborockClient, asyncio.Protocol):
         command = CommandInfoMap.get(method)
         if command is None:
             raise RoborockException(f"No prefix found for {method}")
-        prefix = command.prefix
         request_protocol = 4
         return RoborockMessage(
-            prefix=prefix,
             timestamp=timestamp,
             protocol=request_protocol,
             payload=payload,
         )
 
     async def ping(self):
-        command_info = CommandInfoMap[RoborockCommand.NONE]
-        roborock_message = RoborockMessage(prefix=command_info.prefix, protocol=AP_CONFIG, payload=b"")
+        roborock_message = RoborockMessage(protocol=AP_CONFIG, payload=b"")
         return (await self.send_message(roborock_message))[0]
 
     async def send_command(self, method: RoborockCommand, params: Optional[list | dict] = None):
@@ -92,11 +93,16 @@ class RoborockLocalClient(RoborockClient, asyncio.Protocol):
         return (await self.send_message(roborock_message))[0]
 
     async def async_local_response(self, roborock_message: RoborockMessage):
-        request_id = roborock_message.get_request_id()
-        if request_id is None:
+        method = roborock_message.get_method()
+        request_id: int | None
+        if method and not method.startswith("get"):
             request_id = roborock_message.seq
-        # response_protocol = 5 if roborock_message.prefix == secured_prefix else 4
-        response_protocol = 4
+            response_protocol = 5
+        else:
+            request_id = roborock_message.get_request_id()
+            response_protocol = 4
+        if request_id is None:
+            raise RoborockException(f"Failed build message {roborock_message}")
         (response, err) = await self._async_response(request_id, response_protocol)
         if err:
             raise CommandVacuumError("", err) from err
@@ -116,7 +122,7 @@ class RoborockLocalClient(RoborockClient, asyncio.Protocol):
         if isinstance(roborock_messages, RoborockMessage):
             roborock_messages = [roborock_messages]
         local_key = self.device_info.device.local_key
-        msg = RoborockParser.encode(roborock_messages, local_key)
+        msg = MessageParser.build(roborock_messages, local_key=local_key)
         # Send the command to the Roborock device
         _LOGGER.debug(f"Requesting device with {roborock_messages}")
         self._send_msg_raw(msg)
